@@ -22,8 +22,8 @@ _ssh_verify_certificate_callback (gnutls_session_t session)
   unsigned int status;
   const gnutls_datum_t *cert_list;
   unsigned int cert_list_size;
-  int ret;
-  gnutls_x509_crt_t cert;
+  int ret, type;
+  gnutls_datum_t out;
   const char *hostname;
 
   /* read hostname */
@@ -32,41 +32,30 @@ _ssh_verify_certificate_callback (gnutls_session_t session)
   /* This verification function uses the trusted CAs in the credentials
    * structure. So you must have installed one or more CA certificates.
    */
-  ret = gnutls_certificate_verify_peers2 (session, &status);
+  ret = gnutls_certificate_verify_peers3 (session, hostname, &status);
   if (ret < 0)
     {
       printf ("Error\n");
       return GNUTLS_E_CERTIFICATE_ERROR;
     }
+  
+  type = gnutls_certificate_type_get (session);
 
-  if (status & GNUTLS_CERT_INVALID)
-    printf ("The certificate is not trusted.\n");
-
-  if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-    printf ("The certificate hasn't got a known issuer.\n");
-
-  if (status & GNUTLS_CERT_REVOKED)
-    printf ("The certificate has been revoked.\n");
-
-  if (status & GNUTLS_CERT_EXPIRED)
-    printf ("The certificate has expired\n");
-
-  if (status & GNUTLS_CERT_NOT_ACTIVATED)
-    printf ("The certificate is not yet activated\n");
-
-  /* Up to here the process is the same for X.509 certificates and
-   * OpenPGP keys. From now on X.509 certificates are assumed. This can
-   * be easily extended to work with openpgp keys as well.
-   */
-  if (gnutls_certificate_type_get (session) != GNUTLS_CRT_X509)
-    return GNUTLS_E_CERTIFICATE_ERROR;
-
-  if (gnutls_x509_crt_init (&cert) < 0)
+  ret = gnutls_certificate_verification_status_print( status, type, &out, 0);
+  if (ret < 0)
     {
-      printf ("error in initialization\n");
+      printf ("Error\n");
       return GNUTLS_E_CERTIFICATE_ERROR;
     }
+  
+  printf ("%s", out.data);
+  
+  gnutls_free(out.data);
 
+  if (status != 0) /* Certificate is not trusted */
+      return GNUTLS_E_CERTIFICATE_ERROR;
+
+  /* Do SSH verification */
   cert_list = gnutls_certificate_get_peers (session, &cert_list_size);
   if (cert_list == NULL)
     {
@@ -74,27 +63,9 @@ _ssh_verify_certificate_callback (gnutls_session_t session)
       return GNUTLS_E_CERTIFICATE_ERROR;
     }
 
-  /* This is not a real world example, since we only check the first 
-   * certificate in the given chain.
-   */
-  if (gnutls_x509_crt_import (cert, &cert_list[0], GNUTLS_X509_FMT_DER) < 0)
-    {
-      printf ("error parsing certificate\n");
-      return GNUTLS_E_CERTIFICATE_ERROR;
-    }
-
-  if (!gnutls_x509_crt_check_hostname (cert, hostname))
-    {
-      printf ("The certificate's owner does not match hostname '%s'\n",
-              hostname);
-      status |= GNUTLS_CERT_INVALID;
-    }
-
-  gnutls_x509_crt_deinit (cert);
-  
   /* service may be obtained alternatively using getservbyport() */
   ret = gnutls_verify_stored_pubkey(NULL, NULL, hostname, "https", 
-                                    GNUTLS_CRT_X509, &cert_list[0], 0);
+                                    type, &cert_list[0], 0);
   if (ret == GNUTLS_E_NO_CERTIFICATE_FOUND)
     {
       printf("Host %s is not known.", hostname);
@@ -130,8 +101,7 @@ _ssh_verify_certificate_callback (gnutls_session_t session)
   if (ret != 0)
     {
       ret = gnutls_store_pubkey(NULL, NULL, hostname, "https", 
-                                GNUTLS_CRT_X509, &cert_list[0], 
-                                0, 0);
+                                type, &cert_list[0], 0, 0);
       if (ret < 0)
         printf("gnutls_store_pubkey: %s\n", gnutls_strerror(ret));
     }
