@@ -41,7 +41,6 @@
 #include <gnutls_extensions.h>
 #include <gnutls_constate.h>
 #include <algorithms.h>
-#include <gnutls_state.h>
 
 static int pack_certificate_auth_info (gnutls_session_t,
                                        gnutls_buffer_st * packed_session);
@@ -312,20 +311,20 @@ pack_certificate_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
     {
 
       BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-      BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-      BUFFER_APPEND_PFX (ps, info->dh.generator.data,
+      BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.generator.data,
                          info->dh.generator.size);
-      BUFFER_APPEND_PFX (ps, info->dh.public_key.data,
+      BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data,
                          info->dh.public_key.size);
-      BUFFER_APPEND_PFX (ps, info->rsa_export.modulus.data,
+      BUFFER_APPEND_PFX4 (ps, info->rsa_export.modulus.data,
                          info->rsa_export.modulus.size);
-      BUFFER_APPEND_PFX (ps, info->rsa_export.exponent.data,
+      BUFFER_APPEND_PFX4 (ps, info->rsa_export.exponent.data,
                          info->rsa_export.exponent.size);
 
       BUFFER_APPEND_NUM (ps, info->ncerts);
 
       for (i = 0; i < info->ncerts; i++)
-        BUFFER_APPEND_PFX (ps, info->raw_certificate_list[i].data,
+        BUFFER_APPEND_PFX4 (ps, info->raw_certificate_list[i].data,
                            info->raw_certificate_list[i].size);
     }
 
@@ -445,7 +444,7 @@ pack_srp_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
 
-  BUFFER_APPEND_PFX (ps, username, len);
+  BUFFER_APPEND_PFX4 (ps, username, len);
 
   /* write the real size */
   _gnutls_write_uint32 (ps->length - cur_size, ps->data + size_offset);
@@ -518,15 +517,15 @@ pack_anon_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   size_offset = ps->length;
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
-
+  
   if (info)
     {
       BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-      BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-      BUFFER_APPEND_PFX (ps, info->dh.generator.data,
-                         info->dh.generator.size);
-      BUFFER_APPEND_PFX (ps, info->dh.public_key.data,
-                         info->dh.public_key.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.generator.data,
+                          info->dh.generator.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data,
+                          info->dh.public_key.size);
     }
 
   /* write the real size */
@@ -628,13 +627,13 @@ pack_psk_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
 
-  BUFFER_APPEND_PFX (ps, info->username, username_len);
-  BUFFER_APPEND_PFX (ps, info->hint, hint_len);
+  BUFFER_APPEND_PFX4 (ps, info->username, username_len);
+  BUFFER_APPEND_PFX4 (ps, info->hint, hint_len);
 
   BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-  BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-  BUFFER_APPEND_PFX (ps, info->dh.generator.data, info->dh.generator.size);
-  BUFFER_APPEND_PFX (ps, info->dh.public_key.data, info->dh.public_key.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.generator.data, info->dh.generator.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data, info->dh.public_key.size);
 
   /* write the real size */
   _gnutls_write_uint32 (ps->length - cur_size, ps->data + size_offset);
@@ -901,7 +900,7 @@ gnutls_session_set_premaster (gnutls_session_t session, unsigned int entity,
     return gnutls_assert_val(ret);
 
   session->internals.resumed_security_parameters.compression_method = comp;
-  session->internals.resumed_security_parameters.cert_type = DEFAULT_CERT_TYPE;
+  session->internals.resumed_security_parameters.cert_type = GNUTLS_CRT_UNKNOWN;
   session->internals.resumed_security_parameters.version = version;
 
   if (master->size != GNUTLS_MASTER_SIZE)
