@@ -864,51 +864,6 @@ gnutls_pkcs11_obj_export2 (gnutls_pkcs11_obj_t obj,
   return _gnutls_set_datum(out, obj->raw.data, obj->raw.size);
 }
 
-int
-pkcs11_find_object (struct pkcs11_session_info* sinfo,
-                    struct pin_info_st * pin_info,
-                    ck_object_handle_t * _obj,
-                    struct p11_kit_uri *info, unsigned int flags)
-{
-  int ret;
-  ck_object_handle_t obj;
-  struct ck_attribute *attrs;
-  unsigned long attr_count;
-  unsigned long count;
-  ck_rv_t rv;
-
-  ret = pkcs11_open_session (sinfo, pin_info, info, flags & SESSION_LOGIN);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-
-  attrs = p11_kit_uri_get_attributes (info, &attr_count);
-  rv = pkcs11_find_objects_init (sinfo->module, sinfo->pks, attrs, attr_count);
-  if (rv != CKR_OK)
-    {
-      gnutls_assert ();
-      _gnutls_debug_log ("pk11: FindObjectsInit failed.\n");
-      ret = pkcs11_rv_to_err (rv);
-      goto fail;
-    }
-
-  if (pkcs11_find_objects (sinfo->module, sinfo->pks, &obj, 1, &count) == CKR_OK && count == 1)
-    {
-      *_obj = obj;
-      pkcs11_find_objects_final (sinfo);
-      return 0;
-    }
-
-  ret = GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
-  pkcs11_find_objects_final (sinfo);
-fail:
-  pkcs11_close_session (sinfo);
-
-  return ret;
-}
-
 int
 pkcs11_find_slot (struct ck_function_list ** module, ck_slot_id_t * slot,
                   struct p11_kit_uri *info, struct token_info *_tinfo)
@@ -1193,7 +1148,7 @@ pkcs11_obj_import (ck_object_class_t class, gnutls_pkcs11_obj_t obj,
   return 0;
 }
 
-static int read_pkcs11_pubkey(struct ck_function_list *module,
+int pkcs11_read_pubkey(struct ck_function_list *module,
                               ck_session_handle_t pks, ck_object_handle_t obj,
                               ck_key_type_t key_type, gnutls_datum_t * pubkey)
 {
@@ -1369,7 +1324,7 @@ pkcs11_obj_import_pubkey (struct ck_function_list *module,
     {
       crt->pk_algorithm = mech_to_pk(key_type);
 
-      ret = read_pkcs11_pubkey(module, pks, obj, key_type, crt->pubkey);
+      ret = pkcs11_read_pubkey(module, pks, obj, key_type, crt->pubkey);
       if (ret < 0)
         return gnutls_assert_val(ret);
     }
@@ -1536,7 +1491,6 @@ find_obj_url (struct pkcs11_session_info *sinfo,
 
   while (pkcs11_find_objects (sinfo->module, sinfo->pks, &obj, 1, &count) == CKR_OK && count == 1)
     {
-
       a[0].type = CKA_VALUE;
       a[0].value = cert_data;
       a[0].value_len = MAX_CERT_SIZE;
@@ -2307,6 +2261,16 @@ find_objs (struct pkcs11_session_info* sinfo,
         }
     }
 
+  /* Find objects with given class and type */
+  attr = p11_kit_uri_get_attribute (find_data->info, CKA_CLASS);
+  if (attr)
+    {
+      if(attr->value && attr->value_len == sizeof (ck_object_class_t))
+        class = *((ck_object_class_t*)attr->value);
+      if (class == CKO_CERTIFICATE)
+        type = CKC_X_509;
+    }
+
   cert_data = gnutls_malloc (MAX_CERT_SIZE);
   if (cert_data == NULL)
     {
