@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <c-ctype.h>
 
 #ifdef _WIN32
 # include <windows.h>
@@ -471,3 +472,149 @@ gnutls_x509_trust_list_add_system_trust(gnutls_x509_trust_list_t list,
   return r;
 #endif
 }
+
+#if defined(HAVE_ICONV)
+
+# include <iconv.h>
+
+int _gnutls_ucs2_to_utf8(const void* data, size_t size, gnutls_datum_t *output)
+{
+iconv_t conv;
+int ret;
+size_t orig, dstlen = size*2;
+char* src = (void*)data;
+char* dst = NULL, *pdst;
+
+  if (size == 0)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+    
+  conv = iconv_open("UTF-8", "UTF-16BE");
+  if (conv == (iconv_t)-1)
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+  
+  /* Note that dstlen has enough size for every possible input characters.
+   * (remember the in UTF-16 the characters in data are at most size/2, 
+   *  and we allocate 4 bytes per character).
+   */
+  pdst = dst = gnutls_malloc(dstlen+1);
+  if (dst == NULL)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+      goto fail;
+    }
+
+  orig = dstlen;
+  ret = iconv(conv, &src, &size, &pdst, &dstlen);
+  if (ret == -1)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
+      goto fail;
+    }
+
+  output->data = (void*)dst;
+  output->size = orig-dstlen;
+  output->data[output->size] = 0;
+
+  ret = 0;
+  goto cleanup;
+  
+fail:
+  gnutls_free(dst);
+  
+cleanup:
+  iconv_close(conv);
+  
+  return ret;
+}
+#elif defined(_WIN32)
+#include <winnls.h>
+
+/* Can convert only english */
+int _gnutls_ucs2_to_utf8(const void* data, size_t size, gnutls_datum_t *output)
+{
+int ret;
+unsigned i;
+int len = 0, src_len;
+char* dst = NULL;
+char* src = NULL;
+
+  src_len = size/2;
+
+  src = gnutls_malloc(size);
+  if (src == NULL)
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+  /* convert to LE */
+  for (i=0;i<size;i+=2)
+    {
+      src[i] = ((char*)data)[1+i];
+      src[1+i] = ((char*)data)[i];
+    }
+  
+  ret = WideCharToMultiByte(CP_UTF8, MB_ERR_INVALID_CHARS, (void*)src, src_len,
+                            NULL, 0, NULL, NULL);
+  if (ret == 0)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
+      goto fail;
+    }
+
+  len = ret+1;
+  dst = gnutls_malloc(len);
+  if (dst == NULL)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+      goto fail;
+    }
+  
+  ret = WideCharToMultiByte(CP_UTF8, MB_ERR_INVALID_CHARS, (void*)src, src_len,
+                            dst, len, NULL, NULL);
+  if (ret == 0)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
+      goto fail;
+    }
+  
+  output->data = dst;
+  output->size = ret;
+  ret = 0;
+  goto cleanup;
+  
+fail:
+  gnutls_free(dst);
+
+cleanup:
+  gnutls_free(src);
+  return ret;
+}
+
+#else
+
+/* Can convert only english (ASCII) */
+int _gnutls_ucs2_to_utf8(const void* data, size_t size, gnutls_datum_t *output)
+{
+unsigned int i, j;
+char* dst;
+const char *src = data;
+
+  if (size == 0 || size % 2 != 0)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  dst = gnutls_malloc(size+1);
+  if (dst == NULL)
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+  for (i=j=0;i<size;i+=2,j++)
+    {
+      if (src[i] != 0 || !isascii(src[i+1]))
+        return gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
+      dst[j] = src[i+1];
+    }
+    
+  output->data = (void*)dst;
+  output->size = j;
+  output->data[output->size] = 0;
+
+  return 0;
+}
+#endif
