@@ -28,7 +28,6 @@
 #include <libtasn1.h>
 #include <gnutls_pk.h>
 #include "common.h"
-#include "hash.h"
 #include "verify-high.h"
 
 #include <gnutls/ocsp.h>
@@ -268,7 +267,7 @@ gnutls_ocsp_resp_import (gnutls_ocsp_resp_t resp,
     return GNUTLS_E_SUCCESS;
 
   ret = _gnutls_x509_read_value (resp->resp, "responseBytes.responseType",
-				 &resp->response_type_oid, 0);
+				 &resp->response_type_oid);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -297,7 +296,7 @@ gnutls_ocsp_resp_import (gnutls_ocsp_resp_t resp,
 	}
 
       ret = _gnutls_x509_read_value (resp->resp, "responseBytes.response",
-				     &d, 0);
+				     &d);
       if (ret < 0)
 	{
 	  gnutls_assert ();
@@ -483,7 +482,7 @@ gnutls_ocsp_req_get_cert_id (gnutls_ocsp_req_t req,
   snprintf (name, sizeof (name),
 	    "tbsRequest.requestList.?%u.reqCert.hashAlgorithm.algorithm",
 	    indx + 1);
-  ret = _gnutls_x509_read_value (req->req, name, &sa, 0);
+  ret = _gnutls_x509_read_value (req->req, name, &sa);
   if (ret == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
     return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
   else if (ret < 0)
@@ -507,7 +506,7 @@ gnutls_ocsp_req_get_cert_id (gnutls_ocsp_req_t req,
     {
       snprintf (name, sizeof (name),
 		"tbsRequest.requestList.?%u.reqCert.issuerNameHash", indx + 1);
-      ret = _gnutls_x509_read_value (req->req, name, issuer_name_hash, 0);
+      ret = _gnutls_x509_read_value (req->req, name, issuer_name_hash);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -519,7 +518,7 @@ gnutls_ocsp_req_get_cert_id (gnutls_ocsp_req_t req,
     {
       snprintf (name, sizeof (name),
 		"tbsRequest.requestList.?%u.reqCert.issuerKeyHash", indx + 1);
-      ret = _gnutls_x509_read_value (req->req, name, issuer_key_hash, 0);
+      ret = _gnutls_x509_read_value (req->req, name, issuer_key_hash);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -533,7 +532,7 @@ gnutls_ocsp_req_get_cert_id (gnutls_ocsp_req_t req,
     {
       snprintf (name, sizeof (name),
 		"tbsRequest.requestList.?%u.reqCert.serialNumber", indx + 1);
-      ret = _gnutls_x509_read_value (req->req, name, serial_number, 0);
+      ret = _gnutls_x509_read_value (req->req, name, serial_number);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -719,9 +718,9 @@ gnutls_ocsp_req_add_cert (gnutls_ocsp_req_t req,
   inh.size = inhlen;
   inh.data = inh_buf;
 
-  ret = _gnutls_x509_read_value
+  ret = _gnutls_x509_read_string
     (issuer->cert, "tbsCertificate.subjectPublicKeyInfo.subjectPublicKey",
-     &tmp, 2);
+     &tmp, RV_BIT_STRING);
   if (ret != GNUTLS_E_SUCCESS)
     {
       gnutls_assert ();
@@ -739,7 +738,7 @@ gnutls_ocsp_req_add_cert (gnutls_ocsp_req_t req,
   ikh.data = ikh_buf;
 
   ret = _gnutls_x509_read_value (cert->cert, "tbsCertificate.serialNumber",
-				 &sn, 0);
+				 &sn);
   if (ret != GNUTLS_E_SUCCESS)
     {
       gnutls_assert ();
@@ -822,7 +821,7 @@ gnutls_ocsp_req_get_extension (gnutls_ocsp_req_t req,
     {
       snprintf (name, sizeof (name),
 		"tbsRequest.requestExtensions.?%u.extnID", indx + 1);
-      ret = _gnutls_x509_read_value (req->req, name, oid, 0);
+      ret = _gnutls_x509_read_value (req->req, name, oid);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -834,7 +833,7 @@ gnutls_ocsp_req_get_extension (gnutls_ocsp_req_t req,
     {
       snprintf (name, sizeof (name),
 		"tbsRequest.requestExtensions.?%u.extnValue", indx + 1);
-      ret = _gnutls_x509_read_value (req->req, name, data, 0);
+      ret = _gnutls_x509_read_value (req->req, name, data);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -897,7 +896,6 @@ gnutls_ocsp_req_get_nonce (gnutls_ocsp_req_t req,
 			   gnutls_datum_t *nonce)
 {
   int ret;
-  size_t l = 0;
   gnutls_datum_t tmp;
 
   if (req == NULL || nonce == NULL)
@@ -915,32 +913,16 @@ gnutls_ocsp_req_get_nonce (gnutls_ocsp_req_t req,
       return ret;
     }
 
-  ret = _gnutls_x509_decode_octet_string (NULL, tmp.data, (size_t) tmp.size,
-					  NULL, &l);
-  if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
+  ret = _gnutls_x509_decode_string (NULL, tmp.data, (size_t) tmp.size,
+			            nonce);
+  if (ret < 0)
     {
       gnutls_assert ();
       gnutls_free (tmp.data);
       return ret;
     }
 
-  nonce->data = gnutls_malloc (l);
-  if (nonce->data == NULL)
-    {
-      gnutls_assert ();
-      gnutls_free (tmp.data);
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  ret = _gnutls_x509_decode_octet_string (NULL, tmp.data, (size_t) tmp.size,
-					  nonce->data, &l);
   gnutls_free (tmp.data);
-  if (ret != GNUTLS_E_SUCCESS)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-  nonce->size = l;
 
   return GNUTLS_E_SUCCESS;
 }
@@ -1122,7 +1104,7 @@ gnutls_ocsp_resp_get_response (gnutls_ocsp_resp_t resp,
   if (response_type_oid != NULL)
     {
       ret = _gnutls_x509_read_value (resp->resp, "responseBytes.responseType",
-				     response_type_oid, 0);
+				     response_type_oid);
       if (ret < 0)
 	{
 	  gnutls_assert ();
@@ -1133,7 +1115,7 @@ gnutls_ocsp_resp_get_response (gnutls_ocsp_resp_t resp,
   if (response != NULL)
     {
       ret = _gnutls_x509_read_value (resp->resp, "responseBytes.response",
-				     response, 0);
+				     response);
       if (ret < 0)
 	{
 	  gnutls_assert ();
@@ -1372,7 +1354,6 @@ cleanup:
   return ret;
 }
 
-
 /**
  * gnutls_ocsp_resp_get_single:
  * @resp: should contain a #gnutls_ocsp_resp_t structure
@@ -1389,32 +1370,8 @@ cleanup:
  *
  * This function will return the certificate information of the
  * @indx'ed response in the Basic OCSP Response @resp.  The
- * information returned corresponds to the SingleResponse structure
- * except the final singleExtensions, reproduced here for illustration:
- *
- * <informalexample><programlisting>
- * SingleResponse ::= SEQUENCE {
- *    certID                       CertID,
- *    certStatus                   CertStatus,
- *    thisUpdate                   GeneralizedTime,
- *    nextUpdate         [0]       EXPLICIT GeneralizedTime OPTIONAL,
- *    singleExtensions   [1]       EXPLICIT Extensions OPTIONAL }
- *
- *    CertID          ::=     SEQUENCE {
- *        hashAlgorithm       AlgorithmIdentifier,
- *        issuerNameHash      OCTET STRING, -- Hash of Issuer's DN
- *        issuerKeyHash       OCTET STRING, -- Hash of Issuers public key
- *        serialNumber        CertificateSerialNumber }
- *
- * CertStatus ::= CHOICE {
- *     good                [0]     IMPLICIT NULL,
- *     revoked             [1]     IMPLICIT RevokedInfo,
- *     unknown             [2]     IMPLICIT UnknownInfo }
- * 
- * RevokedInfo ::= SEQUENCE {
- *     revocationTime              GeneralizedTime,
- *     revocationReason    [0]     EXPLICIT CRLReason OPTIONAL }
- * </programlisting></informalexample>
+ * information returned corresponds to the OCSP SingleResponse structure
+ * except the final singleExtensions.
  *
  * Each of the pointers to output variables may be NULL to indicate
  * that the caller is not interested in that value.
@@ -1440,11 +1397,11 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
   gnutls_datum_t sa;
   char name[ASN1_MAX_NAME_SIZE];
   int ret;
-
+  
   snprintf (name, sizeof (name),
 	    "tbsResponseData.responses.?%u.certID.hashAlgorithm.algorithm",
 	    indx + 1);
-  ret = _gnutls_x509_read_value (resp->basicresp, name, &sa, 0);
+  ret = _gnutls_x509_read_value (resp->basicresp, name, &sa);
   if (ret == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
     return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
   else if (ret < 0)
@@ -1470,7 +1427,7 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
 		"tbsResponseData.responses.?%u.certID.issuerNameHash",
 		indx + 1);
       ret = _gnutls_x509_read_value (resp->basicresp, name,
-				     issuer_name_hash, 0);
+				     issuer_name_hash);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -1484,7 +1441,7 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
 		"tbsResponseData.responses.?%u.certID.issuerKeyHash",
 		indx + 1);
       ret = _gnutls_x509_read_value (resp->basicresp, name,
-				     issuer_key_hash, 0);
+				     issuer_key_hash);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -1500,7 +1457,7 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
 		"tbsResponseData.responses.?%u.certID.serialNumber",
 		indx + 1);
       ret = _gnutls_x509_read_value (resp->basicresp, name,
-				     serial_number, 0);
+				     serial_number);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -1517,7 +1474,7 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
       snprintf (name, sizeof (name),
 		"tbsResponseData.responses.?%u.certStatus",
 		indx + 1);
-      ret = _gnutls_x509_read_value (resp->basicresp, name, &sa, 0);
+      ret = _gnutls_x509_read_value (resp->basicresp, name, &sa);
       if (ret == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
 	return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
       else if (ret < 0)
@@ -1599,6 +1556,18 @@ gnutls_ocsp_resp_get_single (gnutls_ocsp_resp_t resp,
     }
 
   /* revocation_reason */
+  if (revocation_reason)
+    {
+      snprintf (name, sizeof (name),
+		"tbsResponseData.responses.?%u.certStatus."
+		"revoked.revocationReason",
+		indx + 1);
+
+      ret = _gnutls_x509_read_uint (resp->basicresp, name,
+                                    revocation_reason);
+      if (ret < 0)
+        *revocation_reason = GNUTLS_X509_CRLREASON_UNSPECIFIED;
+    }
 
   return GNUTLS_E_SUCCESS;
 }
@@ -1669,7 +1638,7 @@ gnutls_ocsp_resp_get_extension (gnutls_ocsp_resp_t resp,
     {
       snprintf (name, sizeof (name),
 		"tbsResponseData.responseExtensions.?%u.extnID", indx + 1);
-      ret = _gnutls_x509_read_value (resp->basicresp, name, oid, 0);
+      ret = _gnutls_x509_read_value (resp->basicresp, name, oid);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -1681,7 +1650,7 @@ gnutls_ocsp_resp_get_extension (gnutls_ocsp_resp_t resp,
     {
       snprintf (name, sizeof (name),
 		"tbsResponseData.responseExtensions.?%u.extnValue", indx + 1);
-      ret = _gnutls_x509_read_value (resp->basicresp, name, data, 0);
+      ret = _gnutls_x509_read_value (resp->basicresp, name, data);
       if (ret != GNUTLS_E_SUCCESS)
 	{
 	  gnutls_assert ();
@@ -1715,7 +1684,6 @@ gnutls_ocsp_resp_get_nonce (gnutls_ocsp_resp_t resp,
 			    gnutls_datum_t *nonce)
 {
   int ret;
-  size_t l = 0;
   gnutls_datum_t tmp;
 
   ret = get_extension (resp->basicresp, "tbsResponseData.responseExtensions",
@@ -1727,32 +1695,16 @@ gnutls_ocsp_resp_get_nonce (gnutls_ocsp_resp_t resp,
       return ret;
     }
 
-  ret = _gnutls_x509_decode_octet_string (NULL, tmp.data, (size_t) tmp.size,
-					  NULL, &l);
-  if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
+  ret = _gnutls_x509_decode_string (NULL, tmp.data, (size_t) tmp.size,
+			   	    nonce);
+  if (ret < 0)
     {
       gnutls_assert ();
       gnutls_free (tmp.data);
       return ret;
     }
 
-  nonce->data = gnutls_malloc (l);
-  if (nonce->data == NULL)
-    {
-      gnutls_assert ();
-      gnutls_free (tmp.data);
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  ret = _gnutls_x509_decode_octet_string (NULL, tmp.data, (size_t) tmp.size,
-					  nonce->data, &l);
   gnutls_free (tmp.data);
-  if (ret != GNUTLS_E_SUCCESS)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-  nonce->size = l;
 
   return GNUTLS_E_SUCCESS;
 }
@@ -1775,7 +1727,7 @@ gnutls_ocsp_resp_get_signature_algorithm (gnutls_ocsp_resp_t resp)
   gnutls_datum_t sa;
 
   ret = _gnutls_x509_read_value (resp->basicresp,
-				 "signatureAlgorithm.algorithm", &sa, 0);
+				 "signatureAlgorithm.algorithm", &sa);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1811,7 +1763,7 @@ gnutls_ocsp_resp_get_signature (gnutls_ocsp_resp_t resp,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  ret = _gnutls_x509_read_value (resp->basicresp, "signature", sig, 2);
+  ret = _gnutls_x509_read_string (resp->basicresp, "signature", sig, RV_BIT_STRING);
   if (ret != GNUTLS_E_SUCCESS)
     {
       gnutls_assert ();
