@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2012 Free Software Foundation, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -28,6 +28,10 @@
 #include <gnutls/abstract.h>
 #include <gnutls/crypto.h>
 
+#ifdef HAVE_DANE
+# include <gnutls/dane.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -53,10 +57,11 @@ static void cmd_parser (int argc, char **argv);
 static void dane_info(const char* host, const char* proto, unsigned int port, 
                       unsigned int ca, unsigned int local, common_info_st * cinfo);
 
+static void dane_check(const char* host, const char* proto, unsigned int port,
+                       common_info_st * cinfo);
+
 FILE *outfile;
-FILE *infile;
 static gnutls_digest_algorithm_t default_dig;
-static unsigned int incert_format, outcert_format;
 
 /* non interactive operation if set
  */
@@ -84,6 +89,8 @@ cmd_parser (int argc, char **argv)
 {
   int ret, privkey_op = 0;
   common_info_st cinfo;
+  const char* proto = "tcp";
+  unsigned int port = 443;
 
   optionProcess( &danetoolOptions, argc, argv);
   
@@ -96,25 +103,6 @@ cmd_parser (int argc, char **argv)
   else
     outfile = stdout;
 
-  if (HAVE_OPT(INFILE))
-    {
-      infile = fopen (OPT_ARG(INFILE), "rb");
-      if (infile == NULL)
-        error (EXIT_FAILURE, errno, "%s", OPT_ARG(INFILE));
-    }
-  else
-    infile = stdin;
-
-  if (HAVE_OPT(INDER) || HAVE_OPT(INRAW))
-    incert_format = GNUTLS_X509_FMT_DER;
-  else
-    incert_format = GNUTLS_X509_FMT_PEM;
-
-  if (HAVE_OPT(OUTDER) || HAVE_OPT(OUTRAW))
-    outcert_format = GNUTLS_X509_FMT_DER;
-  else
-    outcert_format = GNUTLS_X509_FMT_PEM;
-
   default_dig = GNUTLS_DIG_UNKNOWN;
   if (HAVE_OPT(HASH))
     {
@@ -157,6 +145,11 @@ cmd_parser (int argc, char **argv)
 
   memset (&cinfo, 0, sizeof (cinfo));
   
+  if (HAVE_OPT(INDER) || HAVE_OPT(INRAW))
+    cinfo.incert_format = GNUTLS_X509_FMT_DER;
+  else
+    cinfo.incert_format = GNUTLS_X509_FMT_PEM;
+
   if (HAVE_OPT(VERBOSE))
     cinfo.verbose = 1;
   
@@ -166,9 +159,17 @@ cmd_parser (int argc, char **argv)
   if (HAVE_OPT(LOAD_CERTIFICATE))
     cinfo.cert = OPT_ARG(LOAD_CERTIFICATE);
 
+  if (HAVE_OPT(PORT))
+    port = OPT_VALUE_PORT;
+  if (HAVE_OPT(PROTO))
+    proto = OPT_ARG(PROTO);
+
   if (HAVE_OPT(TLSA_RR))
-    dane_info (OPT_ARG(HOST), OPT_ARG(PROTO), OPT_VALUE_PORT, 
+    dane_info (OPT_ARG(HOST), proto, port,
                HAVE_OPT(CA), HAVE_OPT(LOCAL), &cinfo);
+  else if (HAVE_OPT(CHECK))
+    dane_check (OPT_ARG(CHECK), proto, port, 
+                &cinfo);
   else
     USAGE(1);
 
@@ -180,6 +181,110 @@ cmd_parser (int argc, char **argv)
   gnutls_global_deinit ();
 }
 
+static void dane_check(const char* host, const char* proto, unsigned int port,
+                       common_info_st * cinfo)
+{
+#ifdef HAVE_DANE
+dane_state_t s;
+dane_query_t q;
+int ret;
+unsigned entries;
+unsigned int flags = DANE_F_IGNORE_LOCAL_RESOLVER, i;
+unsigned int usage, type, match;
+gnutls_datum_t data, file;
+size_t size;
+
+  if (ENABLED_OPT(LOCAL_DNS))
+    flags = 0;
+
+  printf("Querying %s (%s:%d)...\n", host, proto, port);
+  ret = dane_state_init(&s, flags);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "dane_state_init: %s", dane_strerror (ret));
+  
+  ret = dane_query_tlsa(s, &q, host, proto, port);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "dane_query_tlsa: %s", dane_strerror (ret));
+  
+  entries = dane_query_entries(q);
+  for (i=0;i<entries;i++)
+    {
+      ret = dane_query_data(q, i, &usage, &type, &match, &data);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "dane_query_data: %s", dane_strerror (ret));
+      
+      
+      size = buffer_size;
+      ret = gnutls_hex_encode(&data, (void*)buffer, &size);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "gnutls_hex_encode: %s", dane_strerror (ret));
+
+      if (entries > 1) printf("\nEntry %d:\n", i+1);
+
+      fprintf(outfile, "_%u._%s.%s. IN TLSA ( %.2x %.2x %.2x %s )\n", port, proto, host, usage, type, match, buffer);
+      printf("Certificate usage: %s (%.2x)\n", dane_cert_usage_name(usage), usage);
+      printf("Certificate type:  %s (%.2x)\n", dane_cert_type_name(type), type);
+      printf("Contents:          %s (%.2x)\n", dane_match_type_name(match), match);
+      printf("Data:              %s\n", buffer);
+
+      /* Verify the DANE data */
+      if (cinfo->cert)
+        {
+          gnutls_x509_crt_t *clist;
+          unsigned int clist_size, status;
+          
+          ret = gnutls_load_file(cinfo->cert, &file);
+          if (ret < 0)
+            error (EXIT_FAILURE, 0, "gnutls_load_file: %s", gnutls_strerror (ret));
+        
+          ret = gnutls_x509_crt_list_import2( &clist, &clist_size, &file, cinfo->incert_format, 0);
+          if (ret < 0)
+            error (EXIT_FAILURE, 0, "gnutls_x509_crt_list_import2: %s", gnutls_strerror (ret));
+
+          if (clist_size > 0)
+            {
+              gnutls_datum_t certs[clist_size];
+              gnutls_datum_t out;
+              unsigned int i;
+              
+              for (i=0;i<clist_size;i++)
+                {
+                  ret = gnutls_x509_crt_export2( clist[i], GNUTLS_X509_FMT_DER, &certs[i]);
+                  if (ret < 0)
+                    error (EXIT_FAILURE, 0, "gnutls_x509_crt_export2: %s", gnutls_strerror (ret));
+                }
+
+              ret = dane_verify_crt( s, certs, clist_size, GNUTLS_CRT_X509, 
+                                     host, proto, port, 0, 0, &status);
+              if (ret < 0)
+                error (EXIT_FAILURE, 0, "dane_verify_crt: %s", dane_strerror (ret));
+                
+              ret = dane_verification_status_print(status, &out, 0);
+              if (ret < 0)
+                error (EXIT_FAILURE, 0, "dane_verification_status_print: %s", dane_strerror (ret));
+              
+              printf("\nVerification: %s\n", out.data);
+              gnutls_free(out.data);
+
+              for (i=0;i<clist_size;i++)
+                {
+                  gnutls_free(certs[i].data);
+                  gnutls_x509_crt_deinit(clist[i]);
+                }
+              gnutls_free(clist);
+            }
+        }
+    }
+  
+
+  dane_query_deinit(q);
+  dane_state_deinit(s);
+#else
+  fprintf(stderr, "This functionality was disabled (GnuTLS was not compiled with support for DANE).\n");
+  return;
+#endif
+}
+
 static void dane_info(const char* host, const char* proto, unsigned int port, 
                       unsigned int ca, unsigned int local, common_info_st * cinfo)
 {
