@@ -31,7 +31,7 @@
 #include "certtool-common.h"
 #include <unistd.h>
 #include <string.h>
-#include <p11common.h>
+#include <common.h>
 
 void
 pkcs11_delete (FILE * outfile, const char *url, int batch, unsigned int login,
@@ -77,7 +77,7 @@ pkcs11_list (FILE * outfile, const char *url, int type, unsigned int login,
   gnutls_pkcs11_obj_t *crt_list;
   gnutls_x509_crt_t xcrt;
   unsigned int crt_list_size = 0, i;
-  int ret, otype;
+  int ret;
   char *output;
   int attrs;
   unsigned int obj_flags = 0;
@@ -113,17 +113,9 @@ pkcs11_list (FILE * outfile, const char *url, int type, unsigned int login,
 
   /* give some initial value to avoid asking for the pkcs11 pin twice.
    */
-  crt_list_size = 128;
-  crt_list = malloc (sizeof (*crt_list) * crt_list_size);
-  if (crt_list == NULL)
-    {
-      fprintf (stderr, "Memory error\n");
-      exit (1);
-    }
-
-  ret = gnutls_pkcs11_obj_list_import_url (crt_list, &crt_list_size, url,
+  ret = gnutls_pkcs11_obj_list_import_url2 (&crt_list, &crt_list_size, url,
                                            attrs, obj_flags);
-  if (ret < 0 && ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
+  if (ret < 0)
     {
       fprintf (stderr, "Error in crt_list_import (1): %s\n",
                gnutls_strerror (ret));
@@ -136,26 +128,6 @@ pkcs11_list (FILE * outfile, const char *url, int type, unsigned int login,
       exit (0);
     }
 
-  if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER)
-    {
-      crt_list = realloc (crt_list, sizeof (*crt_list) * crt_list_size);
-      if (crt_list == NULL)
-        {
-          fprintf (stderr, "Memory error\n");
-          exit (1);
-        }
-
-      ret =
-        gnutls_pkcs11_obj_list_import_url (crt_list, &crt_list_size, url,
-                                           attrs, obj_flags);
-      if (ret < 0)
-        {
-          fprintf (stderr, "Error in crt_list_import: %s\n",
-                   gnutls_strerror (ret));
-          exit (1);
-        }
-    }
-
   for (i = 0; i < crt_list_size; i++)
     {
       char buf[128];
@@ -171,9 +143,9 @@ pkcs11_list (FILE * outfile, const char *url, int type, unsigned int login,
 
       fprintf (outfile, "Object %d:\n\tURL: %s\n", i, output);
 
-      otype = gnutls_pkcs11_obj_get_type(crt_list[i]);
       fprintf (outfile, "\tType: %s\n",
-               gnutls_pkcs11_type_get_name (otype));
+               gnutls_pkcs11_type_get_name (gnutls_pkcs11_obj_get_type
+                                            (crt_list[i])));
 
       size = sizeof (buf);
       ret =
@@ -198,6 +170,46 @@ pkcs11_list (FILE * outfile, const char *url, int type, unsigned int login,
           exit (1);
         }
       fprintf (outfile, "\tID: %s\n\n", buf);
+
+
+
+      if (attrs == GNUTLS_PKCS11_OBJ_ATTR_ALL
+          || attrs == GNUTLS_PKCS11_OBJ_ATTR_PRIVKEY)
+        continue;
+
+      ret = gnutls_x509_crt_init (&xcrt);
+      if (ret < 0)
+        {
+          fprintf (stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
+                   gnutls_strerror (ret));
+          exit (1);
+        }
+
+      ret = gnutls_x509_crt_import_pkcs11 (xcrt, crt_list[i]);
+      if (ret < 0)
+        {
+          fprintf (stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
+                   gnutls_strerror (ret));
+          exit (1);
+        }
+
+#if 0
+      size = buffer_size;
+      ret = gnutls_x509_crt_export (xcrt, GNUTLS_X509_FMT_PEM, buffer, &size);
+      if (ret < 0)
+        {
+          fprintf (stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
+                   gnutls_strerror (ret));
+          exit (1);
+        }
+
+      fwrite (buffer, 1, size, outfile);
+      fputs ("\n\n", outfile);
+#endif
+
+      gnutls_x509_crt_deinit (xcrt);
+
+
     }
 
   return;
@@ -512,6 +524,7 @@ pkcs11_generate (FILE * outfile, const char *url, gnutls_pk_algorithm_t pk,
 {
   int ret;
   unsigned int flags = 0;
+  gnutls_datum_t pubkey;
 
   if (login)
     flags = GNUTLS_PKCS11_OBJ_FLAG_LOGIN;
@@ -526,13 +539,21 @@ pkcs11_generate (FILE * outfile, const char *url, gnutls_pk_algorithm_t pk,
   else if (private == 0)
     flags |= GNUTLS_PKCS11_OBJ_FLAG_MARK_NOT_PRIVATE;
 
-  ret = gnutls_pkcs11_privkey_generate(url, pk, bits, label, flags);
+  ret = gnutls_pkcs11_privkey_generate2(url, pk, bits, label, GNUTLS_X509_FMT_PEM,
+                                        &pubkey, flags);
   if (ret < 0)
     {
       fprintf (stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
                gnutls_strerror (ret));
+      if (login == 0)
+        fprintf(stderr, "Note that --login was not specified and it may be required for generation.\n");
+      else if (bits != 1024)
+        fprintf (stderr, "Note that several smart cards do not support arbitrary size keys.\nTry --bits 1024 or 2048.\n");
       exit(1);
     }
+  
+  fwrite (pubkey.data, 1, pubkey.size, outfile);
+  gnutls_free(pubkey.data);
 
   return;
 }
@@ -574,7 +595,7 @@ pkcs11_init (FILE * outfile, const char *url, const char *label,
       exit (1);
     }
 
-  ret = gnutls_pkcs11_token_set_pin (url, NULL, pin, GNUTLS_PKCS11_PIN_USER);
+  ret = gnutls_pkcs11_token_set_pin (url, NULL, pin, GNUTLS_PIN_USER);
   if (ret < 0)
     {
       fprintf (stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
