@@ -333,6 +333,7 @@ LIST_DECLARE_INIT (listener_list, listener_item, listener_free);
 gnutls_session_t initialize_session (int dtls)
 {
   gnutls_session_t session;
+  int ret;
   const char *err;
 
   if (priorities == NULL)
@@ -393,6 +394,17 @@ gnutls_session_t initialize_session (int dtls)
   if (HAVE_OPT (HEARTBEAT))
     gnutls_heartbeat_enable(session, GNUTLS_HB_PEER_ALLOWED_TO_SEND);
 
+#ifdef ENABLE_DTLS_SRTP
+  if (HAVE_OPT (SRTP_PROFILES))
+    {
+      ret = gnutls_srtp_set_profile_direct (session, OPT_ARG(SRTP_PROFILES), &err);
+      if (ret == GNUTLS_E_INVALID_REQUEST) fprintf (stderr, "Syntax error at: %s\n", err);
+      else 
+        fprintf(stderr, "Error in profiles: %s\n", gnutls_strerror(ret));
+      exit (1);
+    }
+#endif
+
   return session;
 }
 
