@@ -50,11 +50,16 @@ socket_recv (const socket_st * socket, void *buffer, int buffer_size)
   int ret;
 
   if (socket->secure)
-    do
-      {
-        ret = gnutls_record_recv (socket->session, buffer, buffer_size);
-      }
-    while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN);
+    {
+      do
+        {
+          ret = gnutls_record_recv (socket->session, buffer, buffer_size);
+          if (ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED)
+            gnutls_heartbeat_pong(socket->session, 0);
+        }
+      while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
+
+    }
   else
     do
       {
